package pyperl;

use strict;
use warnings;

1;

__END__


=encoding utf8

=head1 NAME

pyperl - stub module for pleasing PAUSE

=head1 SYNOPSIS

=head1 DESCRIPTION

=head1 METHODS

=head2

=head2

=cut
