/* DO NOT EDIT!!!
 * Autogenerated by try_perlapi.pl
 */

void 	fake_inittry(void);
int	try_array_len(AV* av);
SV**	try_av_fetch(AV* av, I32 key, I32 lval);
int	try_SvGETMAGIC(SV* sv);
int	try_SvSETMAGIC(SV* sv);
